@@ -401,6 +401,22 @@ def test_cdist_not_symmetric():
     )
 
 
+def test_generators():
+    """
+    We should be able to use a generators as choices in process.extract
+    as long as they are finite.
+    """
+    def generate_choices():
+        choices = ['a', 'Bb', 'CcC']
+        yield from choices
+
+    search = 'aaa'
+    res1 = process_cpp.extract(search, generate_choices())
+    res2 = process_py.extract(search, generate_choices())
+    assert res1 == res2
+    assert len(res1) > 0
+
+
 def test_cdist_pure_python_dtype():
     pytest.importorskip("numpy")
     assert (
